public class Sudoku {
	
	public static int SIZE = 9;
	public static int[][] PUZZLE = {
		{9,0,0,1,0,0,0,0,5},
		{0,0,5,0,9,0,2,0,1},
		{8,0,0,0,4,0,0,0,0},
		{0,0,0,0,8,0,0,0,0},
		{0,0,0,7,0,0,0,0,0},
		{0,0,0,0,2,6,0,0,9},
		{2,0,0,3,0,0,0,0,6},
		{0,0,0,2,0,0,9,0,0},
		{0,0,1,9,0,4,5,7,0}
	};
	
	//Verify if the number can be placed in the position X,Y
	private static boolean canPlaceNumber(int[][] board, int X, int Y, int num) {
		//Verify in same row or same col
		for (int k = 0; k < SIZE; k++) {
			if (board[X][k] == num || board[k][Y] == num) {
				return false;
			}
		}
		
		//Verify in same grid		
		for (int x = 0; x < SIZE; x++) {
			for (int y = 0; y < SIZE; y++) {
				if ((x / 3) == (X / 3) && 
                  (y / 3) == (Y / 3) && 
                     board[x][y] == num) {
                        return false;
				}
			}
		}
 
      return true;
	}

	//Solves the puzzle
	public static boolean solveSudoku(int[][] board, int row, int col) {
		
		//All rows and cols are filled
		if(col >= SIZE && row >= SIZE - 1) return true;

		//End of col, moving to next row
		if(col >= SIZE) {
			row++;
			col = 0;			
		}
		
		if(board[row][col] != 0) 
			return solveSudoku(board, row, col + 1);

		//Check with all possible values
		for(int num = 1; num <= SIZE; num++) {
			//Verify if the number can be placed in the position X,Y
			if(canPlaceNumber(board, row, col, num)) {

				board[row][col] = num;
				
				if (solveSudoku(board, row, col + 1))
					return true;					
			}

			board[row][col] = 0;
		}
		
		return false;
	}
		
	public static void main(String args[]) {
		int[][] board = new int[SIZE][SIZE];
		// fill in the temporary board with the puzzle
		for(int i = 0; i < SIZE; i++) {
			for(int j = 0; j < SIZE; j++) {
				board[i][j] = PUZZLE[i][j];
			}
		}
		
		if(solveSudoku(board, 0, 0)) {
         //Print out results
			for(int i = 0; i < SIZE; i++) {
				for(int j = 0; j < SIZE; j++) {
					System.out.print(" " + board[i][j]);
				}
				System.out.println();
			}
		}
		else {
         //no solution
			System.out.println("No solution found");
		}
	}
}